@@ -19,7 +19,9 @@
 
 #include "pipewire-v4l2.h"
 
+#include <spa/utils/atomic.h>
 #include <spa/utils/result.h>
+#include <spa/utils/keys.h>
 #include <spa/pod/iter.h>
 #include <spa/pod/parser.h>
 #include <spa/pod/filter.h>
@@ -248,8 +250,6 @@ static void update_params(struct file *file)
 		}
 	}
 }
-#define ATOMIC_DEC(s)                   __atomic_sub_fetch(&(s), 1, __ATOMIC_SEQ_CST)
-#define ATOMIC_INC(s)                   __atomic_add_fetch(&(s), 1, __ATOMIC_SEQ_CST)
 
 static struct file *make_file(void)
 {
@@ -301,7 +301,7 @@ static void free_file(struct file *file)
 static void unref_file(struct file *file)
 {
 	pw_log_debug("file:%d ref:%d", file->fd, file->ref);
-	if (ATOMIC_DEC(file->ref) <= 0)
+	if (SPA_ATOMIC_DEC(file->ref) <= 0)
 		free_file(file);
 }
 
@@ -314,7 +314,7 @@ static int add_fd_map(int fd, struct file *file, uint32_t flags)
 		map->fd = fd;
 		map->flags = flags;
 		map->file = file;
-		ATOMIC_INC(file->ref);
+		SPA_ATOMIC_INC(file->ref);
 		pw_log_debug("fd:%d -> file:%d ref:%d", fd, file->fd, file->ref);
 	}
 	pthread_mutex_unlock(&globals.lock);
@@ -349,7 +349,7 @@ static struct fd_map *find_fd_map_unlocked(int fd)
 	struct fd_map *map;
 	pw_array_for_each(map, &globals.fd_maps) {
 		if (map->fd == fd) {
-			ATOMIC_INC(map->file->ref);
+			SPA_ATOMIC_INC(map->file->ref);
 			pw_log_debug("fd:%d find:%d ref:%d", map->fd, fd, map->file->ref);
 			return map;
 		}
@@ -384,7 +384,7 @@ static struct file *find_file_by_dev(uint32_t dev)
 		if (tmp->file->dev_id == dev) {
 			if (tmp->file->closed)
 				tmp->file->fd = tmp->fd;
-			ATOMIC_INC(tmp->file->ref);
+			SPA_ATOMIC_INC(tmp->file->ref);
 			map = tmp;
 			pw_log_debug("dev:%d find:%d ref:%d",
 					tmp->file->dev_id, dev, tmp->file->ref);
@@ -913,7 +913,7 @@ static int v4l2_close(int fd)
 static int vidioc_querycap(struct file *file, struct v4l2_capability *arg)
 {
 	int res = 0;
-	const char *str = NULL;
+	const char *card = NULL, *bus_info = NULL;
 	struct pw_node_info *info;
 
 	if (file->node == NULL)
@@ -922,15 +922,19 @@ static int vidioc_querycap(struct file *file, struct v4l2_capability *arg)
 	info = file->node->info;
 
 	if (info != NULL && info->props != NULL) {
-		str = spa_dict_lookup(info->props, PW_KEY_NODE_DESCRIPTION);
+		card = spa_dict_lookup(info->props, PW_KEY_NODE_DESCRIPTION);
+		bus_info = spa_dict_lookup(info->props, SPA_KEY_API_V4L2_CAP_BUS_INFO);
 	}
-	if (str == NULL)
-		str = DEFAULT_CARD;
+	if (card == NULL)
+		card = DEFAULT_CARD;
 
 	spa_scnprintf((char*)arg->driver, sizeof(arg->driver), "%s", DEFAULT_DRIVER);
-	spa_scnprintf((char*)arg->card, sizeof(arg->card), "%s", str);
-	spa_scnprintf((char*)arg->bus_info, sizeof(arg->bus_info), "platform:%s-%d",
-			DEFAULT_BUS_INFO, file->node->id);
+	spa_scnprintf((char*)arg->card, sizeof(arg->card), "%s", card);
+	if (bus_info == NULL)
+		spa_scnprintf((char*)arg->bus_info, sizeof(arg->bus_info), "platform:%s-%d",
+				DEFAULT_BUS_INFO, file->node->id);
+	else
+		spa_scnprintf((char*)arg->bus_info, sizeof(arg->bus_info), "%s", bus_info);
 
 	arg->version = KERNEL_VERSION(5, 2, 0);
 	arg->device_caps = V4L2_CAP_VIDEO_CAPTURE
@@ -986,7 +990,9 @@ static const struct format_info format_info[] = {
 	MAKE_FORMAT(Y10, video, raw, 2, UNKNOWN),
 	MAKE_FORMAT(Y12, video, raw, 2, UNKNOWN),
 	MAKE_FORMAT(Y16, video, raw, 2, GRAY16_LE),
+#ifdef V4L2_PIX_FMT_Y16_BE
 	MAKE_FORMAT(Y16_BE, video, raw, 2, GRAY16_BE),
+#endif
 	MAKE_FORMAT(Y10BPACK, video, raw, 2, UNKNOWN),
 
 	/* Palette formats */
