@@ -7,6 +7,7 @@
 #include "alsa-ucm.h"
 
 #include <spa/utils/string.h>
+#include <spa/utils/json.h>
 
 int _acp_log_level = 1;
 acp_log_func _acp_log_func;
@@ -385,7 +386,7 @@ static int add_pro_profile(pa_card *impl, uint32_t index)
 							devstr, NULL, &m->sample_spec,
 							&m->channel_map, SND_PCM_STREAM_PLAYBACK,
 							&try_period_size, &try_buffer_size,
-							0, NULL, NULL, false))) {
+							0, NULL, NULL, NULL, NULL, false))) {
 				pa_alsa_init_proplist_pcm(NULL, m->output_proplist, m->output_pcm);
 				pa_proplist_setf(m->output_proplist, "clock.name", "api.alsa.%u", index);
 				pa_proplist_setf(m->output_proplist, "device.profile.pro", "true");
@@ -417,7 +418,7 @@ static int add_pro_profile(pa_card *impl, uint32_t index)
 							devstr, NULL, &m->sample_spec,
 							&m->channel_map, SND_PCM_STREAM_CAPTURE,
 							&try_period_size, &try_buffer_size,
-							0, NULL, NULL, false))) {
+							0, NULL, NULL, NULL, NULL, false))) {
 				pa_alsa_init_proplist_pcm(NULL, m->input_proplist, m->input_pcm);
 				pa_proplist_setf(m->input_proplist, "clock.name", "api.alsa.%u", index);
 				pa_proplist_setf(m->input_proplist, "device.profile.pro", "true");
@@ -449,6 +450,24 @@ static int add_pro_profile(pa_card *impl, uint32_t index)
 	return 0;
 }
 
+static bool contains_string(const char *arr, const char *str)
+{
+	struct spa_json it[2];
+	char v[256];
+
+	if (arr == NULL || str == NULL)
+		return false;
+
+	spa_json_init(&it[0], arr, strlen(arr));
+        if (spa_json_enter_array(&it[0], &it[1]) <= 0)
+                spa_json_init(&it[1], arr, strlen(arr));
+
+	while (spa_json_get_string(&it[1], v, sizeof(v)) > 0) {
+		if (spa_streq(v, str))
+			return true;
+	}
+	return false;
+}
 
 static void add_profiles(pa_card *impl)
 {
@@ -459,6 +478,7 @@ static void add_profiles(pa_card *impl)
 	pa_alsa_device *dev;
 	int n_profiles, n_ports, n_devices;
 	uint32_t idx;
+	const char *arr;
 
 	n_devices = 0;
 	pa_dynarray_init(&impl->out.devices, device_free);
@@ -490,8 +510,8 @@ static void add_profiles(pa_card *impl)
 					pa_dynarray_append(&impl->out.devices, dev);
 				}
 				if (impl->use_ucm) {
-					if (m->ucm_context.ucm_devices) {
-						pa_alsa_ucm_add_ports_combination(NULL, &m->ucm_context,
+					if (m->ucm_context.ucm_device) {
+						pa_alsa_ucm_add_port(NULL, &m->ucm_context,
 							true, impl->ports, ap, NULL);
 						pa_alsa_ucm_add_ports(&dev->ports, m->proplist, &m->ucm_context,
 							true, impl, dev->pcm_handle, impl->profile_set->ignore_dB);
@@ -514,8 +534,8 @@ static void add_profiles(pa_card *impl)
 				}
 
 				if (impl->use_ucm) {
-					if (m->ucm_context.ucm_devices) {
-						pa_alsa_ucm_add_ports_combination(NULL, &m->ucm_context,
+					if (m->ucm_context.ucm_device) {
+						pa_alsa_ucm_add_port(NULL, &m->ucm_context,
 							false, impl->ports, ap, NULL);
 						pa_alsa_ucm_add_ports(&dev->ports, m->proplist, &m->ucm_context,
 							false, impl, dev->pcm_handle, impl->profile_set->ignore_dB);
@@ -561,7 +581,10 @@ static void add_profiles(pa_card *impl)
 		dev->device.ports = dev->port_array.array.data;
 		dev->device.n_ports = pa_dynarray_size(&dev->port_array);
 	}
+	arr = pa_proplist_gets(impl->proplist, "api.acp.hidden-ports");
 	PA_HASHMAP_FOREACH(dp, impl->ports, state) {
+		if (contains_string(arr, dp->name))
+			dp->port.flags |= ACP_PORT_HIDDEN;
 		dp->port.devices = dp->devices.array.data;
 		dp->port.n_devices = pa_dynarray_size(&dp->devices);
 	}
@@ -570,7 +593,10 @@ static void add_profiles(pa_card *impl)
 
 	n_profiles = 0;
 	pa_dynarray_init(&impl->out.profiles, NULL);
+	arr = pa_proplist_gets(impl->proplist, "api.acp.hidden-profiles");
 	PA_HASHMAP_FOREACH(cp, impl->profiles, state) {
+		if (contains_string(arr, cp->name))
+			cp->flags |= ACP_PROFILE_HIDDEN;
 		cp->index = n_profiles++;
 		pa_dynarray_append(&impl->out.profiles, cp);
 	}
@@ -649,7 +675,7 @@ struct temp_port_avail {
 static int report_jack_state(snd_mixer_elem_t *melem, unsigned int mask)
 {
 	pa_card *impl = snd_mixer_elem_get_callback_private(melem);
-	snd_hctl_elem_t *elem = snd_mixer_elem_get_private(melem);
+	snd_hctl_elem_t **_elem = snd_mixer_elem_get_private(melem), *elem;
 	snd_ctl_elem_value_t *elem_value;
 	bool plugged_in, any_input_port_available;
 	void *state;
@@ -659,6 +685,8 @@ static int report_jack_state(snd_mixer_elem_t *melem, unsigned int mask)
 	enum acp_available active_available = ACP_AVAILABLE_UNKNOWN;
 	size_t size;
 
+	pa_assert(_elem);
+	elem = *_elem;
 #if 0
 	/* Changing the jack state may cause a port change, and a port change will
 	 * make the sink or source change the mixer settings. If there are multiple
@@ -927,13 +955,17 @@ static pa_device_port* find_port_with_eld_device(pa_card *impl, int device)
 static int hdmi_eld_changed(snd_mixer_elem_t *melem, unsigned int mask)
 {
 	pa_card *impl = snd_mixer_elem_get_callback_private(melem);
-	snd_hctl_elem_t *elem = snd_mixer_elem_get_private(melem);
-	int device = snd_hctl_elem_get_device(elem);
+	snd_hctl_elem_t **_elem = snd_mixer_elem_get_private(melem), *elem;
+	int device;
 	const char *old_monitor_name;
 	pa_device_port *p;
 	pa_hdmi_eld eld;
 	bool changed = false;
 
+	pa_assert(_elem);
+	elem = *_elem;
+	device = snd_hctl_elem_get_device(elem);
+
 	if (mask == SND_CTL_EVENT_MASK_REMOVE)
 		return 0;
 
@@ -1021,6 +1053,9 @@ uint32_t acp_card_find_best_profile_index(struct acp_card *card, const char *nam
 	for (i = 0; i < card->n_profiles; i++) {
 		struct acp_card_profile *p = profiles[i];
 
+		if (SPA_FLAG_IS_SET(p->flags, ACP_PROFILE_HIDDEN))
+			continue;
+
 		if (name) {
 			if (spa_streq(name, p->name))
 				best = i;
@@ -1108,6 +1143,14 @@ static int read_volume(pa_alsa_device *dev)
 	uint32_t i;
 	int res;
 
+	if (dev->ucm_context) {
+		if (!dev->active_port)
+			return 0;
+		pa_alsa_ucm_port_data *data = PA_DEVICE_PORT_DATA(dev->active_port);
+		if (pa_alsa_ucm_port_device_status(data) <= 0)
+			return 0;
+	}
+
 	if (!dev->mixer_handle)
 		return 0;
 
@@ -1143,6 +1186,14 @@ static void set_volume(pa_alsa_device *dev, const pa_cvolume *v)
 	if (v != &dev->real_volume)
 		dev->real_volume = *v;
 
+	if (dev->ucm_context) {
+		if (!dev->active_port)
+			return;
+		pa_alsa_ucm_port_data *data = PA_DEVICE_PORT_DATA(dev->active_port);
+		if (pa_alsa_ucm_port_device_status(data) <= 0)
+			return;
+	}
+
 	if (!dev->mixer_handle)
 		return;
 
@@ -1196,6 +1247,14 @@ static int read_mute(pa_alsa_device *dev)
 	bool mute;
 	int res;
 
+	if (dev->ucm_context) {
+		if (!dev->active_port)
+			return 0;
+		pa_alsa_ucm_port_data *data = PA_DEVICE_PORT_DATA(dev->active_port);
+		if (pa_alsa_ucm_port_device_status(data) <= 0)
+			return 0;
+	}
+
 	if (!dev->mixer_handle)
 		return 0;
 
@@ -1218,6 +1277,14 @@ static void set_mute(pa_alsa_device *dev, bool mute)
 {
 	dev->muted = mute;
 
+	if (dev->ucm_context) {
+		if (!dev->active_port)
+			return;
+		pa_alsa_ucm_port_data *data = PA_DEVICE_PORT_DATA(dev->active_port);
+		if (pa_alsa_ucm_port_device_status(data) <= 0)
+			return;
+	}
+
 	if (!dev->mixer_handle)
 		return;
 
@@ -1268,7 +1335,7 @@ static void mixer_volume_init(pa_card *impl, pa_alsa_device *dev)
 		pa_log_info("Using hardware volume control. Hardware dB scale %s.",
 				dev->mixer_path->has_dB ? "supported" : "not supported");
 	}
-	dev->device.base_volume = pa_sw_volume_to_linear(dev->base_volume);
+	dev->device.base_volume = (float)pa_sw_volume_to_linear(dev->base_volume);
 	dev->device.volume_step = 1.0f / dev->n_volume_steps;
 
 	if (impl->soft_mixer || !dev->mixer_path || !dev->mixer_path->has_mute) {
@@ -1295,8 +1362,7 @@ static int setup_mixer(pa_card *impl, pa_alsa_device *dev, bool ignore_dB)
 	* will be NULL, but the UCM device enable sequence will still need to be
 	* executed. */
 	if (dev->active_port && dev->ucm_context) {
-		if ((res = pa_alsa_ucm_set_port(dev->ucm_context, dev->active_port,
-					dev->direction == PA_ALSA_DIRECTION_OUTPUT)) < 0)
+		if ((res = pa_alsa_ucm_set_port(dev->ucm_context, dev->active_port)) < 0)
 			return res;
 	}
 
@@ -1441,9 +1507,11 @@ int acp_card_set_profile(struct acp_card *card, uint32_t new_index, uint32_t fla
 	if (new_index >= card->n_profiles)
 		return -EINVAL;
 
-	op = old_index != ACP_INVALID_INDEX ? (pa_alsa_profile*)profiles[old_index] : NULL;
 	np = (pa_alsa_profile*)profiles[new_index];
+	if (SPA_FLAG_IS_SET(np->profile.flags, ACP_PROFILE_HIDDEN))
+		return -EINVAL;
 
+	op = old_index != ACP_INVALID_INDEX ? (pa_alsa_profile*)profiles[old_index] : NULL;
 	if (op == np)
 		return 0;
 
@@ -1469,10 +1537,19 @@ int acp_card_set_profile(struct acp_card *card, uint32_t new_index, uint32_t fla
 	}
 
 	/* if UCM is available for this card then update the verb */
-	if (impl->use_ucm && !(np->profile.flags & ACP_PROFILE_PRO)) {
-		if ((res = pa_alsa_ucm_set_profile(&impl->ucm, impl,
-		    np->profile.flags & ACP_PROFILE_OFF ? NULL : np->profile.name,
-		    op ? op->profile.name : NULL)) < 0) {
+	if (impl->use_ucm) {
+		if (np->profile.flags & ACP_PROFILE_OFF) {
+			if ((res = pa_alsa_ucm_set_profile(&impl->ucm, impl, NULL, op)) < 0)
+				return res;
+		} else if (np->profile.flags & ACP_PROFILE_PRO) {
+			const char *verb = find_best_verb(impl);
+			if ((res = pa_alsa_ucm_set_profile(&impl->ucm, impl, NULL, op)) < 0)
+				return res;
+			if ((res = snd_use_case_set(impl->ucm.ucm_mgr, "_verb", verb)) < 0) {
+				pa_log_error("error setting verb: %s", snd_strerror(res));
+				return res;
+			}
+		} else if ((res = pa_alsa_ucm_set_profile(&impl->ucm, impl, np, op)) < 0) {
 			return res;
 		}
 	}
@@ -1481,8 +1558,8 @@ int acp_card_set_profile(struct acp_card *card, uint32_t new_index, uint32_t fla
 		PA_IDXSET_FOREACH(am, np->output_mappings, idx) {
 			if (impl->use_ucm) {
 				/* Update ports priorities */
-				if (am->ucm_context.ucm_devices) {
-					pa_alsa_ucm_add_ports_combination(am->output.ports, &am->ucm_context,
+				if (am->ucm_context.ucm_device) {
+					pa_alsa_ucm_add_port(am->output.ports, &am->ucm_context,
 						true, impl->ports, np, NULL);
 				}
 			}
@@ -1494,8 +1571,8 @@ int acp_card_set_profile(struct acp_card *card, uint32_t new_index, uint32_t fla
 		PA_IDXSET_FOREACH(am, np->input_mappings, idx) {
 			if (impl->use_ucm) {
 				/* Update ports priorities */
-				if (am->ucm_context.ucm_devices) {
-					pa_alsa_ucm_add_ports_combination(am->input.ports, &am->ucm_context,
+				if (am->ucm_context.ucm_device) {
+					pa_alsa_ucm_add_port(am->input.ports, &am->ucm_context,
 						false, impl->ports, np, NULL);
 				}
 			}
@@ -1548,16 +1625,6 @@ static void prune_singleton_availability_groups(pa_hashmap *ports) {
     pa_hashmap_free(group_counts);
 }
 
-static const char *acp_dict_lookup(const struct acp_dict *dict, const char *key)
-{
-	const struct acp_dict_item *it;
-	acp_dict_for_each(it, dict) {
-		if (spa_streq(key, it->key))
-			return it->value;
-	}
-	return NULL;
-}
-
 struct acp_card *acp_card_new(uint32_t index, const struct acp_dict *props)
 {
 	pa_card *impl;
@@ -1831,6 +1898,9 @@ uint32_t acp_device_find_best_port_index(struct acp_device *dev, const char *nam
 	for (i = 0; i < dev->n_ports; i++) {
 		struct acp_port *p = ports[i];
 
+		if (SPA_FLAG_IS_SET(p->flags, ACP_PORT_HIDDEN))
+			continue;
+
 		if (name) {
 			if (spa_streq(name, p->name))
 				best = i;
@@ -1870,6 +1940,8 @@ int acp_device_set_port(struct acp_device *dev, uint32_t port_index, uint32_t fl
 	p = (pa_device_port*)impl->card.ports[port_index];
 	if (!pa_hashmap_get(d->ports, p->name))
 		return -EINVAL;
+	if (SPA_FLAG_IS_SET(p->port.flags, ACP_PORT_HIDDEN))
+		return -EINVAL;
 
 	p->port.flags = ACP_PORT_ACTIVE | flags;
 	if (p == old)
@@ -1886,8 +1958,7 @@ int acp_device_set_port(struct acp_device *dev, uint32_t port_index, uint32_t fl
 		mixer_volume_init(impl, d);
 
 		sync_mixer(d, p);
-		res = pa_alsa_ucm_set_port(d->ucm_context, p,
-					dev->direction == ACP_DIRECTION_PLAYBACK);
+		res = pa_alsa_ucm_set_port(d->ucm_context, p);
 	} else {
 		pa_alsa_port_data *data;
 
@@ -1951,7 +2022,7 @@ static int get_volume(pa_cvolume *v, float *volume, uint32_t n_volume)
 	if (v->channels == 0)
 		return -EIO;
 	for (i = 0; i < n_volume; i++)
-		volume[i] = pa_sw_volume_to_linear(v->values[i % v->channels]);
+		volume[i] = (float)pa_sw_volume_to_linear(v->values[i % v->channels]);
 	return 0;
 }
 
