@@ -1,26 +1,6 @@
-/* Spa MIDI node
- *
- * Copyright © 2022 Pauli Virtanen
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* Spa MIDI node */
+/* SPDX-FileCopyrightText: Copyright © 2022 Pauli Virtanen */
+/* SPDX-License-Identifier: MIT */
 
 #include <unistd.h>
 #include <stddef.h>
@@ -61,7 +41,7 @@
 
 #include "bluez5-interface-gen.h"
 
-static struct spa_log_topic log_topic = SPA_LOG_TOPIC(0, "spa.bluez5.midi.node");
+SPA_LOG_TOPIC_DEFINE_STATIC(log_topic, "spa.bluez5.midi.node");
 #undef SPA_LOG_TOPIC_DEFAULT
 #define SPA_LOG_TOPIC_DEFAULT &log_topic
 
@@ -618,7 +598,7 @@ again:
 				-SPA_CLAMP(err_nsec, -20*SPA_NSEC_PER_MSEC, 20*SPA_NSEC_PER_MSEC)
 				* this->rate / SPA_NSEC_PER_SEC);
 		tcorr = SPA_MIN(device_elapsed, SPA_NSEC_PER_SEC) * (corr - 1);
-		sync->device_time += tcorr;
+		sync->device_time += (uint64_t)tcorr;
 
 		/* reset if too much off */
 		if (err_nsec < -50 * SPA_NSEC_PER_MSEC ||
@@ -888,13 +868,20 @@ static void on_timeout(struct spa_source *source)
 	spa_log_trace(this->log, "%p: timer %"PRIu64" %"PRIu64"", this,
 			now_time, now_time - prev_time);
 
-	update_position(this);
+	if (SPA_LIKELY(this->position)) {
+		this->duration = this->position->clock.target_duration;
+		this->rate = this->position->clock.target_rate.denom;
+	} else {
+		this->duration = 1024;
+		this->rate = 48000;
+	}
 
 	this->next_time = now_time + this->duration * SPA_NSEC_PER_SEC / this->rate;
 
 	if (SPA_LIKELY(this->clock)) {
 		this->clock->nsec = now_time;
-		this->clock->position += this->duration;
+		this->clock->rate = this->clock->target_rate;
+		this->clock->position += this->clock->duration;
 		this->clock->duration = this->duration;
 		this->clock->rate_diff = 1.0f;
 		this->clock->next_nsec = this->next_time;
@@ -1123,17 +1110,10 @@ static int do_remove_source(struct spa_loop *loop,
 		void *user_data)
 {
 	struct impl *this = user_data;
-	struct itimerspec ts;
 
 	if (this->timer_source.loop)
 		spa_loop_remove_source(this->data_loop, &this->timer_source);
-
-	ts.it_value.tv_sec = 0;
-	ts.it_value.tv_nsec = 0;
-	ts.it_interval.tv_sec = 0;
-	ts.it_interval.tv_nsec = 0;
-	spa_system_timerfd_settime(this->data_system, this->timerfd, 0, &ts, NULL);
-
+	set_timeout(this, 0);
 	return 0;
 }
 
@@ -1237,7 +1217,6 @@ static int do_reassign_follower(struct spa_loop *loop,
 		void *user_data)
 {
 	struct impl *this = user_data;
-
 	set_timers(this);
 	return 0;
 }
@@ -2032,11 +2011,13 @@ impl_init(const struct spa_handle_factory *factory,
 			SPA_DICT_ITEM_INIT(SPA_KEY_FORMAT_DSP, "8 bit raw midi"),
 			SPA_DICT_ITEM_INIT(SPA_KEY_PORT_NAME, "in"),
 			SPA_DICT_ITEM_INIT(SPA_KEY_PORT_ALIAS, "in"),
+			SPA_DICT_ITEM_INIT(SPA_KEY_PORT_GROUP, "group.0"),
 		};
 		static const struct spa_dict_item out_port_items[] = {
 			SPA_DICT_ITEM_INIT(SPA_KEY_FORMAT_DSP, "8 bit raw midi"),
 			SPA_DICT_ITEM_INIT(SPA_KEY_PORT_NAME, "out"),
 			SPA_DICT_ITEM_INIT(SPA_KEY_PORT_ALIAS, "out"),
+			SPA_DICT_ITEM_INIT(SPA_KEY_PORT_GROUP, "group.0"),
 		};
 		static const struct spa_dict in_port_props = SPA_DICT_INIT_ARRAY(in_port_items);
 		static const struct spa_dict out_port_props = SPA_DICT_INIT_ARRAY(out_port_items);
@@ -2054,7 +2035,9 @@ impl_init(const struct spa_handle_factory *factory,
 			SPA_PORT_CHANGE_MASK_PARAMS;
 		port->info = SPA_PORT_INFO_INIT();
 		port->info.change_mask = SPA_PORT_CHANGE_MASK_FLAGS;
-		port->info.flags = SPA_PORT_FLAG_LIVE;
+		port->info.flags = SPA_PORT_FLAG_LIVE |
+			SPA_PORT_FLAG_PHYSICAL |
+			SPA_PORT_FLAG_TERMINAL;
 		port->params[IDX_EnumFormat] = SPA_PARAM_INFO(SPA_PARAM_EnumFormat, SPA_PARAM_INFO_READ);
 		port->params[IDX_Meta] = SPA_PARAM_INFO(SPA_PARAM_Meta, SPA_PARAM_INFO_READ);
 		port->params[IDX_IO] = SPA_PARAM_INFO(SPA_PARAM_IO, SPA_PARAM_INFO_READ);
