@@ -31,7 +31,8 @@ PW_LOG_TOPIC_EXTERN(mod_topic);
 #define MAX_BUFFERS	64
 #define MAX_METAS	16u
 #define MAX_DATAS	64u
-#define AREA_SIZE	(4096u / sizeof(struct spa_io_buffers))
+#define AREA_SLOT	(sizeof(struct spa_io_async_buffers))
+#define AREA_SIZE	(4096u / AREA_SLOT)
 #define MAX_AREAS	32
 
 #define CHECK_FREE_PORT(impl,d,p)	(p <= pw_map_get_size(&impl->ports[d]) && !CHECK_PORT(impl,d,p))
@@ -70,6 +71,7 @@ struct port {
 	uint32_t id;
 
 	struct spa_node mix_node;
+	struct spa_hook_list mix_hooks;
 
 	struct spa_port_info info;
 	struct pw_properties *properties;
@@ -583,12 +585,11 @@ impl_node_remove_port(void *object, enum spa_direction direction, uint32_t port_
 }
 
 static int
-impl_node_port_enum_params(void *object, int seq,
+node_port_enum_params(struct impl *impl, int seq,
 			   enum spa_direction direction, uint32_t port_id,
 			   uint32_t id, uint32_t start, uint32_t num,
-			   const struct spa_pod *filter)
+			   const struct spa_pod *filter, struct spa_hook_list *hooks)
 {
-	struct impl *impl = object;
 	struct port *port;
 	uint8_t buffer[1024];
 	struct spa_pod_dynamic_builder b;
@@ -628,7 +629,7 @@ impl_node_port_enum_params(void *object, int seq,
 		spa_pod_dynamic_builder_init(&b, buffer, sizeof(buffer), 4096);
 		if (spa_pod_filter(&b.b, &result.param, param, filter) == 0) {
 			pw_log_debug("%p: %d param %u", impl, seq, result.index);
-			spa_node_emit_result(&impl->hooks, seq, 0, SPA_RESULT_TYPE_NODE_PARAMS, &result);
+			spa_node_emit_result(hooks, seq, 0, SPA_RESULT_TYPE_NODE_PARAMS, &result);
 			count++;
 		}
 		spa_pod_dynamic_builder_clean(&b);
@@ -639,6 +640,17 @@ impl_node_port_enum_params(void *object, int seq,
 	return found ? 0 : -ENOENT;
 }
 
+static int
+impl_node_port_enum_params(void *object, int seq,
+			   enum spa_direction direction, uint32_t port_id,
+			   uint32_t id, uint32_t start, uint32_t num,
+			   const struct spa_pod *filter)
+{
+	struct impl *impl = object;
+	return node_port_enum_params(impl, seq, direction, port_id, id,
+			start, num, filter, &impl->hooks);
+}
+
 static int clear_buffers_cb(void *item, void *data)
 {
 	if (item)
@@ -855,9 +867,12 @@ do_port_use_buffers(struct impl *impl,
 			switch (d->type) {
 			case SPA_DATA_DmaBuf:
 			case SPA_DATA_MemFd:
+			case SPA_DATA_SyncObj:
 			{
 				uint32_t flags = PW_MEMBLOCK_FLAG_DONT_CLOSE;
 
+				if (!(d->flags & SPA_DATA_FLAG_MAPPABLE))
+					flags |= PW_MEMBLOCK_FLAG_UNMAPPABLE;
 				if (d->flags & SPA_DATA_FLAG_READABLE)
 					flags |= PW_MEMBLOCK_FLAG_READABLE;
 				if (d->flags & SPA_DATA_FLAG_WRITABLE)
@@ -1084,16 +1099,27 @@ static int client_node_port_buffers(void *data,
 		for (j = 0; j < b->buffer.n_datas; j++) {
 			struct spa_chunk *oldchunk = oldbuf->datas[j].chunk;
 			struct spa_data *d = &newbuf->datas[j];
+			uint32_t flags = d->flags;
+
+			if (d->type == SPA_DATA_MemFd &&
+			    !SPA_FLAG_IS_SET(flags, SPA_DATA_FLAG_MAPPABLE)) {
+				spa_log_debug(impl->log, "buffer:%d data:%d has non mappable MemFd, "
+						"fixing to ensure backwards compatibility.",
+						i, j);
+				flags |= SPA_DATA_FLAG_MAPPABLE;
+			}
 
 			/* overwrite everything except the chunk */
 			oldbuf->datas[j] = *d;
+			oldbuf->datas[j].flags = flags;
 			oldbuf->datas[j].chunk = oldchunk;
 
 			b->datas[j].type = d->type;
+			b->datas[j].flags = flags;
 			b->datas[j].fd = d->fd;
 
 			spa_log_debug(impl->log, " data %d type:%d fl:%08x fd:%d, offs:%d max:%d",
-					j, d->type, d->flags, (int) d->fd, d->mapoffset,
+					j, d->type, flags, (int) d->fd, d->mapoffset,
 					d->maxsize);
 		}
 	}
@@ -1166,12 +1192,8 @@ static const struct spa_node_methods impl_node = {
 
 static int
 impl_init(struct impl *impl,
-	  struct spa_dict *info,
-	  const struct spa_support *support,
-	  uint32_t n_support)
+	  struct spa_dict *info)
 {
-	impl->log = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Log);
-
 	impl->node.iface = SPA_INTERFACE_INIT(
 			SPA_TYPE_INTERFACE_Node,
 			SPA_VERSION_NODE,
@@ -1325,8 +1347,6 @@ void pw_impl_client_node_registered(struct pw_impl_client_node *this, struct pw_
 					  0,
 					  sizeof(struct pw_node_activation));
 
-	node_peer_added(impl, node);
-
 	if (impl->bind_node_id) {
 		pw_global_bind(global, client, PW_PERM_ALL,
 				impl->bind_node_version, impl->bind_node_id);
@@ -1338,12 +1358,12 @@ static int add_area(struct impl *impl)
 	size_t size;
 	struct pw_memblock *area;
 
-	size = sizeof(struct spa_io_buffers) * AREA_SIZE;
+	size = AREA_SLOT * AREA_SIZE;
 
 	area = pw_mempool_alloc(impl->context_pool,
 			PW_MEMBLOCK_FLAG_READWRITE |
-			PW_MEMBLOCK_FLAG_MAP |
-			PW_MEMBLOCK_FLAG_SEAL,
+			PW_MEMBLOCK_FLAG_SEAL |
+			PW_MEMBLOCK_FLAG_MAP,
 			SPA_DATA_MemFd, size);
 	if (area == NULL)
                 return -errno;
@@ -1424,6 +1444,7 @@ static int port_init_mix(void *data, struct pw_impl_port_mix *mix)
 	struct mix *m;
 	uint32_t idx, pos, len;
 	struct pw_memblock *area;
+	struct spa_io_async_buffers *ab;
 
 	if ((m = create_mix(port, mix->port.port_id)) == NULL)
 		return -ENOMEM;
@@ -1447,9 +1468,12 @@ static int port_init_mix(void *data, struct pw_impl_port_mix *mix)
 	}
 	area = *pw_array_get_unchecked(&impl->io_areas, idx, struct pw_memblock*);
 
-	mix->io = SPA_PTROFF(area->map->ptr,
-			pos * sizeof(struct spa_io_buffers), void);
-	*mix->io = SPA_IO_BUFFERS_INIT;
+	ab = SPA_PTROFF(area->map->ptr, pos * AREA_SLOT, void);
+	mix->io_data = ab;
+	mix->io[0] = &ab->buffers[0];
+	mix->io[1] = &ab->buffers[1];
+	*mix->io[0] = SPA_IO_BUFFERS_INIT;
+	*mix->io[1] = SPA_IO_BUFFERS_INIT;
 
 	m->peer_id = mix->peer_id;
 	m->impl_mix_id = mix->id;
@@ -1459,8 +1483,8 @@ static int port_init_mix(void *data, struct pw_impl_port_mix *mix)
 					 mix->port.direction, mix->p->port_id,
 					 mix->port.port_id, mix->peer_id, NULL);
 
-	pw_log_debug("%p: init mix id:%d io:%p base:%p", impl,
-			mix->id, mix->io, area->map->ptr);
+	pw_log_debug("%p: init mix id:%d io:%p/%p base:%p", impl,
+			mix->id, mix->io[0], mix->io[1], area->map->ptr);
 
 	return 0;
 no_mem:
@@ -1504,19 +1528,28 @@ static const struct pw_impl_port_implementation port_impl = {
 	.release_mix = port_release_mix,
 };
 
+static int
+impl_mix_add_listener(void *object, struct spa_hook *listener,
+		const struct spa_node_events *events, void *data)
+{
+	struct port *port = object;
+	spa_hook_list_append(&port->mix_hooks, listener, events, data);
+	return 0;
+}
+
 static int
 impl_mix_port_enum_params(void *object, int seq,
-			   enum spa_direction direction, uint32_t port_id,
-			   uint32_t id, uint32_t start, uint32_t num,
-			   const struct spa_pod *filter)
+		enum spa_direction direction, uint32_t port_id,
+		uint32_t id, uint32_t start, uint32_t num,
+		const struct spa_pod *filter)
 {
 	struct port *port = object;
 
 	if (port->direction != direction)
 		return -ENOTSUP;
 
-	return impl_node_port_enum_params(port->impl, seq, direction, port->id,
-			id, start, num, filter);
+	return node_port_enum_params(port->impl, seq, direction, port->id,
+			id, start, num, filter, &port->mix_hooks);
 }
 
 static int
@@ -1572,11 +1605,24 @@ static int impl_mix_port_set_io(void *object,
 	if (mix == NULL)
 		return -EINVAL;
 
-	if (id == SPA_IO_Buffers) {
+	switch (id) {
+	case SPA_IO_Buffers:
 		if (data && size >= sizeof(struct spa_io_buffers))
-			mix->io = data;
+			mix->io[0] = mix->io[1] = data;
 		else
-			mix->io = NULL;
+			mix->io[0] = mix->io[1] = NULL;
+		break;
+	case SPA_IO_AsyncBuffers:
+		if (data && size >= sizeof(struct spa_io_async_buffers)) {
+			struct spa_io_async_buffers *ab = data;
+			mix->io[0] = &ab->buffers[0];
+			mix->io[1] = &ab->buffers[1];
+		}
+		else
+			mix->io[0] = mix->io[1] = NULL;
+		break;
+	default:
+		break;
 	}
 	return do_port_set_io(impl,
 			      direction, port->port_id, mix->port.port_id,
@@ -1597,6 +1643,7 @@ static int impl_mix_process(void *object)
 
 static const struct spa_node_methods impl_port_mix = {
 	SPA_VERSION_NODE_METHODS,
+	.add_listener = impl_mix_add_listener,
 	.port_enum_params = impl_mix_port_enum_params,
 	.port_set_param = impl_mix_port_set_param,
 	.add_port = impl_mix_add_port,
@@ -1621,6 +1668,7 @@ static void node_port_init(void *data, struct pw_impl_port *port)
 	p->id = port->port_id;
 	p->impl = impl;
 	pw_map_init(&p->mix, 2, 2);
+	spa_hook_list_init(&p->mix_hooks);
 	p->mix_node.iface = SPA_INTERFACE_INIT(
 			SPA_TYPE_INTERFACE_Node,
 			SPA_VERSION_NODE,
@@ -1657,16 +1705,6 @@ static void node_port_removed(void *data, struct pw_impl_port *port)
 	clear_port(impl, p);
 }
 
-static void node_driver_changed(void *data, struct pw_impl_node *old, struct pw_impl_node *driver)
-{
-	struct impl *impl = data;
-
-	pw_log_debug("%p: driver changed %p -> %p", impl, old, driver);
-
-	node_peer_removed(data, old);
-	node_peer_added(data, driver);
-}
-
 static const struct pw_impl_node_events node_events = {
 	PW_VERSION_IMPL_NODE_EVENTS,
 	.free = node_free,
@@ -1676,7 +1714,6 @@ static const struct pw_impl_node_events node_events = {
 	.port_removed = node_port_removed,
 	.peer_added = node_peer_added,
 	.peer_removed = node_peer_removed,
-	.driver_changed = node_driver_changed,
 };
 
 static const struct pw_resource_events resource_events = {
@@ -1705,8 +1742,6 @@ struct pw_impl_client_node *pw_impl_client_node_new(struct pw_resource *resource
 	struct pw_impl_client_node *this;
 	struct pw_impl_client *client = pw_resource_get_client(resource);
 	struct pw_context *context = pw_impl_client_get_context(client);
-	const struct spa_support *support;
-	uint32_t n_support;
 	int res;
 
 	impl = calloc(1, sizeof(struct impl));
@@ -1731,8 +1766,8 @@ struct pw_impl_client_node *pw_impl_client_node_new(struct pw_resource *resource
 	impl->data_source.fd = -1;
 	pw_log_debug("%p: new", &impl->node);
 
-	support = pw_context_get_support(impl->context, &n_support);
-	impl_init(impl, NULL, support, n_support);
+	impl_init(impl, NULL);
+	impl->log = pw_log_get();
 	impl->resource = resource;
 	impl->client = client;
 	impl->client_pool = pw_impl_client_get_mempool(client);
@@ -1764,6 +1799,11 @@ struct pw_impl_client_node *pw_impl_client_node_new(struct pw_resource *resource
 
 	this->node->remote = true;
 	this->flags = 0;
+	if (resource->version < PW_VERSION_CLIENT_NODE) {
+		pw_log_warn("detected old client version %d", resource->version);
+		if (resource->version < 6)
+			this->node->rt.target.activation->client_version = 0;
+	}
 
 	pw_resource_add_listener(this->resource,
 				&impl->resource_listener,
