@@ -199,7 +199,7 @@ struct impl {
 
 	struct spa_dll dll;
 	float max_error;
-	float corr;
+	double corr;
 
 	uint64_t next_time;
 	unsigned int have_sync:1;
@@ -244,7 +244,7 @@ static void on_timeout(void *d, uint64_t expirations)
 	pw_log_debug("timeout %"PRIu64, duration);
 
 	current_time = impl->next_time;
-	impl->next_time += duration / impl->corr * 1e9 / rate;
+	impl->next_time += (uint64_t)(duration / impl->corr * 1e9 / rate);
 	avail = spa_ringbuffer_get_read_index(&impl->ring, &index);
 
 	if (SPA_LIKELY(pos)) {
@@ -376,7 +376,7 @@ static void update_rate(struct impl *impl, uint32_t filled)
 
 	if (!impl->driving) {
 		SPA_FLAG_SET(impl->rate_match->flags, SPA_IO_RATE_MATCH_FLAG_ACTIVE);
-		impl->rate_match->rate = 1.0f / impl->corr;
+		impl->rate_match->rate = 1.0 / impl->corr;
 	}
 }
 
@@ -416,6 +416,10 @@ static void capture_stream_process(void *data)
 		}
 		impl->have_sync = false;
 	}
+	if (avail > (int32_t)(impl->target_buffer * 3)) {
+		pw_log_warn("resync %d > %u", avail, (int32_t)(impl->target_buffer * 3));
+		impl->have_sync = false;
+	}
 	if (avail > (int32_t)RINGBUFFER_SIZE) {
 		index += avail - impl->target_buffer;
 		avail = impl->target_buffer;
@@ -727,6 +731,8 @@ static void impl_destroy(struct impl *impl)
 	if (impl->fd >= 0)
 		close(impl->fd);
 
+	pw_context_release_loop(impl->context, impl->data_loop);
+
 	pw_properties_free(impl->stream_props);
 	pw_properties_free(impl->props);
 
@@ -853,7 +859,6 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	struct pw_properties *props = NULL;
 	struct impl *impl;
 	const char *str, *media_class = NULL;
-	struct pw_data_loop *data_loop;
 	int res;
 
 	PW_LOG_TOPIC_INIT(mod_topic);
@@ -887,8 +892,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	impl->module = module;
 	impl->context = context;
 	impl->main_loop = pw_context_get_main_loop(context);
-	data_loop = pw_context_get_data_loop(context);
-	impl->data_loop = pw_data_loop_get_loop(data_loop);
+	impl->data_loop = pw_context_acquire_loop(context, &props->dict);
 
 	if ((str = pw_properties_get(props, "tunnel.mode")) == NULL)
 		str = "playback";
@@ -919,6 +923,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	if ((str = pw_properties_get(props, "tunnel.may-pause")) != NULL)
 		impl->may_pause = spa_atob(str);
 
+	pw_properties_set(props, PW_KEY_NODE_LOOP_NAME, impl->data_loop->name);
 	if (pw_properties_get(props, PW_KEY_NODE_VIRTUAL) == NULL)
 		pw_properties_set(props, PW_KEY_NODE_VIRTUAL, "true");
 	if (pw_properties_get(props, PW_KEY_MEDIA_CLASS) == NULL)
@@ -927,6 +932,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	if ((str = pw_properties_get(props, "stream.props")) != NULL)
 		pw_properties_update_string(impl->stream_props, str, strlen(str));
 
+	copy_props(impl, props, PW_KEY_NODE_LOOP_NAME);
 	copy_props(impl, props, PW_KEY_AUDIO_FORMAT);
 	copy_props(impl, props, PW_KEY_AUDIO_RATE);
 	copy_props(impl, props, PW_KEY_AUDIO_CHANNELS);
@@ -952,7 +958,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	if (impl->info.rate != 0 &&
 	    pw_properties_get(props, PW_KEY_NODE_RATE) == NULL)
 		pw_properties_setf(props, PW_KEY_NODE_RATE,
-				"1/%u", impl->info.rate),
+				"1/%u", impl->info.rate);
 
 	copy_props(impl, props, PW_KEY_NODE_RATE);
 
